/*
 *  Program     :  SCTP CLIENT ONE TO MANY
 *  Author      :  Andres Medina 
 *  Last review :  04/04/2012 
 *  Summary     :
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/sctp.h>
#include <string.h>

#define PORT 12345
#define MSG_SIZE 50000
#define NUMBER_OF_MESSAGES 1000 

/*
 *           FUNCION MAIN
 */
int main(int argc, char * argv[]){

    int sd, ret, i;
    struct sockaddr_in addr;
    struct sctp_initmsg initmsg;
    struct sctp_paddrparams heartbeat;
    struct sctp_rtoinfo rtoinfo;
    char buffer[MSG_SIZE];
    socklen_t to_len;

    memset(&buffer, '1', MSG_SIZE);
    memset(&initmsg,    0,   sizeof(struct sctp_initmsg));
    memset(&heartbeat,  0,   sizeof(struct sctp_paddrparams));
    memset(&rtoinfo,    0, sizeof(struct sctp_rtoinfo));

    /*  Chequeo de sintaxis y errores en los argumentos ingresados */
    if ( argc != 2 || (inet_addr(argv[1]) == -1) ){
        printf("Usage: client [IP ADDRESS in form xxx.xxx.xxx.xxx]\n");
        exit(-1);
    } 

    /* Se setean los valores de familia, puerto y direccion IP del socket */
    if ( inet_aton(argv[1], &addr.sin_addr) == 0 ){
        printf("Error: invalid IP address\n");
        exit(-1);
    }
    addr.sin_port = htons(PORT);
    addr.sin_family = AF_INET; 

    /* Se setean los valores de configuracion de SCTP */
    initmsg.sinit_num_ostreams = 3;
    initmsg.sinit_max_instreams = 1;
    initmsg.sinit_max_attempts = 1;

    heartbeat.spp_flags = SPP_HB_ENABLE;
    heartbeat.spp_hbinterval = 5000;
    heartbeat.spp_pathmaxrxt = 1;

    rtoinfo.srto_max = 2000;

    /* Se crea el socket */
    if ( (sd = socket(AF_INET, SOCK_SEQPACKET, IPPROTO_SCTP )) < 0 ){
        printf("Error: cannot create socket\n");
        exit(-1);
    }

   /*Configure Heartbeats*/
    if((ret = setsockopt(sd, SOL_SCTP, SCTP_PEER_ADDR_PARAMS , &heartbeat, sizeof(heartbeat))) != 0)
        perror("setsockopt");

    printf("Heartbeat interval %d\n", heartbeat.spp_hbinterval);

    /*Set rto_max*/
    if((ret = setsockopt(sd, SOL_SCTP, SCTP_RTOINFO , &rtoinfo, sizeof(rtoinfo))) != 0)
        perror("setsockopt");

    /*Set SCTP Init Message*/
    if((ret = setsockopt(sd, SOL_SCTP, SCTP_INITMSG, &initmsg, sizeof(initmsg))) != 0)
        perror("setsockopt");
 
    to_len = sizeof(addr);


    /* Se envia un chunk al servidor cada 1 segundo hasta completar N chunks */
    for( i = 0; i < NUMBER_OF_MESSAGES; i++ ){
    
        printf("Sending data chunk #%d ...", (i+1));

        if( (ret = sctp_sendmsg(sd, buffer, MSG_SIZE, (struct sockaddr*)&addr, to_len, 0, 0, (i%initmsg.sinit_num_ostreams), 0, 0 )) == -1){
            printf("Error: no se puede enviar el chunk al servidor\n");
            exit(-1);
        }

        printf("Sent %d bytes to peer\n",ret);

        sleep(1);
    }

    /* Cerramos el socket en caso de terminar el programa */
    if( close(sd) < 0 ){
        printf("Error: cannot close the socket file descriptor\n");
        exit(-1);
    }

    return 0;
}
