/*
 *  Program     :  SCTP SERVER ONE TO MANY
 *  Author      :  Andres Medina 
 *  Last review :  04/04/2012 
 *  Summary     :
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/sctp.h>
#include <errno.h>
#include <sys/types.h>
#include <string.h>

#define PORT 12345
#define BUFF_SIZE (1 << 16)

/*
 *           FUNCION MAIN
 */
int main(int argc, char * argv[]){

    int flags, sd, ret, i, addr_count, reuse = 1;
    struct sockaddr_in addr;
    struct sockaddr_in *laddr[10];
    socklen_t from_len, client_len;
    char buffer[BUFF_SIZE];
    struct sctp_paddrparams heartbeat;
    struct sctp_rtoinfo rtoinfo;

    memset(&heartbeat,  0, sizeof(struct sctp_paddrparams));
    memset(&rtoinfo,    0, sizeof(struct sctp_rtoinfo));


    /* Se setean los valores de familia, puerto y direccion IP del socket */
    addr.sin_addr.s_addr = htons(INADDR_ANY);
    addr.sin_port = htons(PORT);
    addr.sin_family = AF_INET; 

    /* Se crea el socket */
    if ( (sd = socket(AF_INET, SOCK_SEQPACKET, IPPROTO_SCTP )) < 0 ){
        printf("Error: cannot create socket\n");
        exit(-1);
    }

    /* Se setea la configuracion de SCTP */
    heartbeat.spp_flags = SPP_HB_ENABLE;
    heartbeat.spp_hbinterval = 5000;
    heartbeat.spp_pathmaxrxt = 3;

    rtoinfo.srto_max = 2000;
 

    from_len = sizeof(addr);

    /*Set Heartbeats*/
    if(setsockopt(sd, SOL_SCTP, SCTP_PEER_ADDR_PARAMS , &heartbeat, sizeof(heartbeat)) != 0)
        perror("setsockopt");

    /*Set rto_max*/
    if(setsockopt(sd, SOL_SCTP, SCTP_RTOINFO , &rtoinfo, sizeof(rtoinfo)) != 0)
        perror("setsockopt");

    /*Set the Reuse of Address*/
    if(setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(int))< 0)
        perror("setsockopt");
  
  
    /* Asociamos el socket a la direccion IP */
    if ( bind(sd, (struct sockaddr*) &addr, from_len) < 0){
        printf("Error: cannot bind the socket to the given IP address\n");
        exit(-1);
    }

    /* Se imprimen las IP local asociadas al socket */
    addr_count = sctp_getladdrs(sd, 0, (struct sockaddr **) laddr);
    printf("Addresses binded: %d\n", addr_count);
    for(i = 0; i < addr_count; i++)
         printf("Address %d: %s:%d\n", i +1, inet_ntoa((*laddr)[i].sin_addr), ntohs((*laddr)[i].sin_port));
    sctp_freeladdrs((struct sockaddr*)*laddr);


    /* Escuchamos el puerto para recibir conexiones */
    if ( (listen(sd, 1)) < 0 ){
        printf("Error: listen failed\n");
        exit(-1);
    }

    printf("Waiting for conections ...\n");

    /* Se reciben los mensajes enviados por el cliente */
    while(1){

        flags = 0;      



        ret = sctp_recvmsg(sd, buffer, BUFF_SIZE, (struct sockaddr*)&addr, &from_len, NULL, &flags);

        printf("%d bytes sent by Client: %s:%d.", ret, inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));        

        if ( getsockname(sd, (struct sockaddr*)&addr, &from_len) < 0){
            printf("Error: getsockname failed\n");
            exit(-1);
        }
 
        printf(" Received in IP address:  %s:%d\n", inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));
        

    }   


    /* Cerramos el socket en caso de terminar el programa */
    if( close(sd) < 0 ){
        printf("Error: cannot close the socket file descriptor\n");
        exit(-1);
    }

    return 0;
}
